==> spit-0.5.1.tar.gz: lib/src/model.dart <==
import 'dart:io';

import 'package:markdown/markdown.dart';
import 'package:path/path.dart' as path;

import 'template.dart';

class Documentation {
  const Documentation({
    required this.name,
    required this.root,
    this.author,
  });

  final String name;
  final String? author;
  final Page root;

  String render() {
    final menu = '<ul>${root.renderMenu(0)}</ul>';
    final content = root.renderContent();
    return pageTemplate(
      name: name,
      menu: menu,
      content: content,
      author: author,
    );
  }

  static Future<Documentation> load({
    required Directory directory,
    String? name,
    String? author,
  }) async {
    final currentName = name ?? path.basename(Directory.current.parent.path);
    return Documentation(
      name: currentName,
      author: author,
      root: (await Page.load(directory, currentName)),
    );
  }
}

class Page {
  const Page({
    required this.name,
    required this.path,
    required this.content,
    required this.children,
  });

  Page withName(String name) => Page(
        name: name,
        path: path,
        content: content,
        children: children,
      );

  String get id => path.isEmpty ? '-home' : Uri.encodeComponent(path.join('-'));
  final String name;
  final List<String> path;
  final String content;
  final List<Page> children;

  String renderMenu(int scope) {
    final buffer = StringBuffer();
    if (scope > 0) {
      buffer.write(
          '<li class="${scope <= 1 ? 'primary' : 'secondary'}"><a id="menu--$id" href="#$id">$name</a>');
    }
    if (children.isNotEmpty) {
      if (scope > 0) buffer.write('<ul>');
      for (var item in children) {
        buffer.write(item.renderMenu(scope + 1));
      }
      if (scope > 0) buffer.write('</ul>');
    }
    if (scope > 0) {
      buffer.write('</li>');
    }
    return buffer.toString();
  }

  String renderContent() {
    final buffer = StringBuffer(
        '<div id="content-$id" class="page" style="display:none">$content</div>');

    for (var item in children) {
      buffer.write(item.renderContent());
    }

    return buffer.toString();
  }

  static Future<Page> load(Directory directory, String name) {
    return _loadDirectory(
      [],
      name,
      directory,
    );
  }
}

Future<Page> _loadDirectory(
    List<String> currentPath, String name, Directory directory) async {
  final index = File('index.md');
  final hasContent = index.existsSync();
  final children = <Page>[];
  for (var item in await directory.list().toList()) {
    if (item is Directory) {
      final hasMdFiles = await item
          .list(recursive: true)
          .any((x) => path.extension(x.path) == '.md');
      if (hasMdFiles) {
        final itemName = path.basenameWithoutExtension(item.path);
        children.add(await _loadDirectory(<String>[
          ...currentPath,
          itemName,
        ], itemName, item));
      }
    } else if (item is File) {
      if (path.extension(item.path) == '.md' &&
          path.basename(item.path).toLowerCase() != 'index.md') {
        children.add(await _loadFile(<String>[
          ...currentPath,
          path.basenameWithoutExtension(item.path),
        ], item));
      }
    }
  }
  final content = hasContent
      ? await index.readAsString()
      : '# $name\n\n${children.map((x) => '* [${x.name}](#${x.id})').join('\n')}';

  return Page(
    name: name,
    path: currentPath,
    content: renderMarkdown(
      currentPath,
      content,
    ),
    children: children,
  );
}

Future<Page> _loadFile(List<String> currentPath, File file) async {
  final name = path.basenameWithoutExtension(file.path);
  final content = await file.readAsString();

  return Page(
    name: name,
    path: currentPath,
    content: renderMarkdown(
      currentPath.take(currentPath.length - 1).toList(),
      content,
    ),
    children: const <Page>[],
  );
}

String renderMarkdown(
  List<String> path,
  String markdown, {
  Iterable<BlockSyntax> blockSyntaxes = const [],
  Iterable<InlineSyntax> inlineSyntaxes = const [],
  ExtensionSet? extensionSet,
  Resolver? linkResolver,
  Resolver? imageLinkResolver,
  bool inlineOnly = false,
}) {
  var document = Document(
    blockSyntaxes: blockSyntaxes,
    inlineSyntaxes: inlineSyntaxes,
    extensionSet: extensionSet,
    linkResolver: linkResolver,
    imageLinkResolver: imageLinkResolver,
  );

  // Replace windows line endings with unix line endings, and split.
  var lines = markdown.replaceAll('\r\n', '\n').split('\n');
  final nodes = document.parseLines(lines);

  _replaceByAbsoluteImageSrcs(path, nodes);

  return renderToHtml(nodes) + '\n';
}

void _replaceByAbsoluteImageSrcs(List<String> path, List<Node> nodes) {
  for (var node in nodes) {
    if (node is Element) {
      _replaceByAbsoluteImageSrc(path, node);
    }
  }
}

void _replaceByAbsoluteImageSrc(List<String> path, Element node) {
  if (node.tag == 'img') {
    node.attributes['src'] = [
      'pages',
      ...path,
      node.attributes['src'],
    ].join('/');
  }
  final children = node.children;
  if (children != null) {
    for (var child in children) {
      if (child is Element) _replaceByAbsoluteImageSrc(path, child);
    }
  }
}

==> spit-0.5.2.tar.gz: lib/src/model.dart <==
import 'dart:io';

import 'package:markdown/markdown.dart';
import 'package:path/path.dart' as path;

import 'template.dart';

class Documentation {
  const Documentation({
    required this.name,
    required this.root,
    this.author,
  });

  final String name;
  final String? author;
  final Page root;

  String render() {
    final menu = '<ul>${root.renderMenu(0)}</ul>';
    final content = root.renderContent();
    return pageTemplate(
      name: name,
      menu: menu,
      content: content,
      author: author,
    );
  }

  static Future<Documentation> load({
    required Directory directory,
    String? name,
    String? author,
  }) async {
    final currentName = name ?? path.basename(Directory.current.parent.path);
    return Documentation(
      name: currentName,
      author: author,
      root: (await Page.load(directory, currentName)),
    );
  }
}

class Page {
  const Page({
    required this.name,
    required this.path,
    required this.content,
    required this.children,
  });

  Page withName(String name) => Page(
        name: name,
        path: path,
        content: content,
        children: children,
      );

  String get id => path.isEmpty ? '-home' : Uri.encodeComponent(path.join('-'));
  final String name;
  final List<String> path;
  final String content;
  final List<Page> children;

  String renderMenu(int scope) {
    final buffer = StringBuffer();
    if (scope > 0) {
      buffer.write(
          '<li class="${scope <= 1 ? 'primary' : 'secondary'}"><a id="menu--$id" href="#$id">$name</a>');
    }
    if (children.isNotEmpty) {
      if (scope > 0) buffer.write('<ul>');
      for (var item in children) {
        buffer.write(item.renderMenu(scope + 1));
      }
      if (scope > 0) buffer.write('</ul>');
    }
    if (scope > 0) {
      buffer.write('</li>');
    }
    return buffer.toString();
  }

  String renderContent() {
    final buffer = StringBuffer(
        '<div id="content-$id" class="page" style="display:none">$content</div>');

    for (var item in children) {
      buffer.write(item.renderContent());
    }

    return buffer.toString();
  }

  static Future<Page> load(Directory directory, String name) {
    return _loadDirectory(
      [],
      name,
      directory,
    );
  }
}

Future<Page> _loadDirectory(
    List<String> currentPath, String name, Directory directory) async {
  final index = File(path.join(directory.path, 'index.md'));
  final hasContent = index.existsSync();
  final children = <Page>[];
  for (var item in await directory.list().toList()) {
    if (item is Directory) {
      final hasMdFiles = await item
          .list(recursive: true)
          .any((x) => path.extension(x.path) == '.md');
      if (hasMdFiles) {
        final itemName = path.basenameWithoutExtension(item.path);
        children.add(await _loadDirectory(<String>[
          ...currentPath,
          itemName,
        ], itemName, item));
      }
    } else if (item is File) {
      if (path.extension(item.path) == '.md' &&
          path.basename(item.path).toLowerCase() != 'index.md') {
        children.add(await _loadFile(<String>[
          ...currentPath,
          path.basenameWithoutExtension(item.path),
        ], item));
      }
    }
  }
  final content = hasContent
      ? await index.readAsString()
      : '# $name\n\n${children.map((x) => '* [${x.name}](#${x.id})').join('\n')}';

  return Page(
    name: name,
    path: currentPath,
    content: renderMarkdown(
      currentPath,
      content,
    ),
    children: children,
  );
}

Future<Page> _loadFile(List<String> currentPath, File file) async {
  final name = path.basenameWithoutExtension(file.path);
  final content = await file.readAsString();

  return Page(
    name: name,
    path: currentPath,
    content: renderMarkdown(
      currentPath.take(currentPath.length - 1).toList(),
      content,
    ),
    children: const <Page>[],
  );
}

String renderMarkdown(
  List<String> path,
  String markdown, {
  Iterable<BlockSyntax> blockSyntaxes = const [],
  Iterable<InlineSyntax> inlineSyntaxes = const [],
  ExtensionSet? extensionSet,
  Resolver? linkResolver,
  Resolver? imageLinkResolver,
  bool inlineOnly = false,
}) {
  var document = Document(
    blockSyntaxes: blockSyntaxes,
    inlineSyntaxes: inlineSyntaxes,
    extensionSet: extensionSet,
    linkResolver: linkResolver,
    imageLinkResolver: imageLinkResolver,
  );

  // Replace windows line endings with unix line endings, and split.
  var lines = markdown.replaceAll('\r\n', '\n').split('\n');
  final nodes = document.parseLines(lines);

  _replaceByAbsoluteImageSrcs(path, nodes);

  return renderToHtml(nodes) + '\n';
}

void _replaceByAbsoluteImageSrcs(List<String> path, List<Node> nodes) {
  for (var node in nodes) {
    if (node is Element) {
      _replaceByAbsoluteImageSrc(path, node);
    }
  }
}

void _replaceByAbsoluteImageSrc(List<String> path, Element node) {
  if (node.tag == 'img') {
    node.attributes['src'] = [
      'pages',
      ...path,
      node.attributes['src'],
    ].join('/');
  }
  final children = node.children;
  if (children != null) {
    for (var child in children) {
      if (child is Element) _replaceByAbsoluteImageSrc(path, child);
    }
  }
}
